import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/NetError.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///转让设备
class MakeOverWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;

  const MakeOverWidget(
      {Key? key, required this.deviceName, required this.deviceId})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return MakeOverState();
  }
}

class MakeOverState extends State<MakeOverWidget> {
  final _userId = TextEditingController();


  ///转让设备
  _makeOver(){
    try{
      EasyLoading.show(status: S.current.device_inTranslation);
      DeviceManage().translationDevice(_userId.value.text, widget.deviceId);
      EasyLoading.showSuccess(S.current.device_successHint( S.of(context).device_makeOverDevice),duration: const Duration(seconds: 1));
      Future.delayed(const Duration(seconds: 1),() {
        Navigator.pop(context);
        Navigator.pop(context);
      });
    }on PlatformException catch(e){
      EasyLoading.dismiss();
      var error = NetError.error(int.parse(e.code));
      EasyLoading.showError(error.error);
    }
  }


  @override
  void dispose() {
    super.dispose();
    _userId.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black12,
      appBar: AppBar(
          title: Text(
              S.of(context).device_makeOverDevice + "(${widget.deviceName})",
              style: const TextStyle(fontSize: 16))),
      body: ListView(children: [
        Container(
          margin: const EdgeInsets.fromLTRB(0, 10, 0, 0),
          padding: const EdgeInsets.fromLTRB(10,18, 10, 18),
          decoration: const BoxDecoration(
            color: Colors.white,
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Text(S.of(context).device_deviceName,
                  style: const TextStyle(fontSize: 16)),
              Text(widget.deviceName, style: const TextStyle(fontSize: 14)),
            ],
          ),
        ),

        Container(
          margin: const EdgeInsets.fromLTRB(0, 10, 0, 0),
          padding: const EdgeInsets.fromLTRB(10, 18, 10, 18),
          decoration: const BoxDecoration(
            color: Colors.white,
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Text(S.of(context).device_deviceId,
                  style: const TextStyle(fontSize: 16)),
              Text(widget.deviceId, style: const TextStyle(fontSize: 14)),
            ],
          ),
        ),
        //
        Container(
          margin: const EdgeInsets.fromLTRB(0, 10, 0, 50),
          padding: const EdgeInsets.fromLTRB(14, 5, 10, 10),
          decoration: const BoxDecoration(
            color: Colors.white,
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Text(S.of(context).device_userId,
                  style: const TextStyle(fontSize: 16)),
              Expanded(
                child: TextField(
                  controller: _userId,
                  onChanged: (v) {
                    setState(() {});
                  },
                  textAlign: TextAlign.end,
                  style: const TextStyle(fontSize: 14),
                  decoration: InputDecoration(
                      contentPadding: const EdgeInsets.fromLTRB(10, 0, 0, 0),
                      border: InputBorder.none,
                      hintText: S.of(context).device_recipientHint),
                ),
              ),
            ],
          ),
        ),

        AnimatedSwitcher(
            duration: const Duration(milliseconds: 200),
            transitionBuilder: (Widget child, Animation<double> animation) {
              return ScaleTransition(
              scale: animation,
              child: child);
            },
            child: _userId.value.text != ""
                ? MaterialButton(
                    onPressed: _makeOver,
                    color: Colors.blue,
                    minWidth: double.infinity,
                    padding: const EdgeInsets.all(20),
                    child: Text(S.of(context).device_makeOverDevice,
                        style: const TextStyle(color: Colors.white)),
                  )
                : const SizedBox(width: 0, height: 0))
      ]),
    );
  }
}
