import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/makeOver/MakeOver.dart';
import 'package:witsystem_device_manage/pages/home/share/DeviceShare.dart';
import 'package:witsystem_device_manage/pages/home/share/ShareType.dart';
import 'package:witsystem_device_manage/pages/home/slock/card/CardList.dart';
import 'package:witsystem_device_manage/pages/home/slock/mark/MarkList.dart';
import 'package:witsystem_device_manage/pages/home/slock/password/PasswordList.dart';
import 'package:witsystem_device_manage/pages/home/slock/record/LockRecord.dart';
import 'package:witsystem_device_manage/pages/home/slock/setup/SlockSetup.dart';
import 'package:witsystem_device_manage/pages/home/slock/widget/UnlockWidget.dart';
import 'package:witsystem_device_manage/pages/home/userManage/UserManageWidget.dart';
import 'package:witsystem_device_manage/pages/widget/HomeListData.dart';
import 'package:witsystem_device_manage/pages/widget/HomeTemplet.dart';
import 'package:witsystem_device_plugin/code/DeviceFunctionType.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';

///门锁主页
class SlcokHome extends StatefulWidget {
  final String deviceId;
  final String deviceName;

  //进入页面是否立即开锁
  final bool immediatelyOpenLock;

  /// const SlcokHome({Key? key}) :super(key: key);
  const SlcokHome(
      {Key? key,
      required this.deviceId,
      required this.deviceName,
      this.immediatelyOpenLock = false})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return SlcokHomeState();
  }
}

class SlcokHomeState extends State<SlcokHome> {
  List<HomeListData>? list = [];
  StreamSubscription? _bleStateListen;
  Timer? _timer;
  String _hint = S.current.device_disconnectHint; //提示
  bool _exeLock = false; //true 代表开锁 false 代表只连接
  @override
  void initState() {
    super.initState();
    _exeLock = widget.immediatelyOpenLock;
    _initListAll();
  }

  ///初始化所有功能列表
  _initListAll() {
    list = [
      HomeListData(
          icon: WitsIcon.yonghu,
          name: S.current.device_userManage,
          widget: UserManageWidget(
              deviceId: widget.deviceId, deviceName: widget.deviceName)),
      HomeListData(
          icon: WitsIcon.jianpan,
          name: S.current.device_passwordManage,
          hardware: DeviceFunctionType.bk,
          widget: PasswordList(
              deviceName: widget.deviceName, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.zhiwen,
          name: S.current.device_markManage,
          hardware: DeviceFunctionType.mark,
          widget: MarkListWidget(
              deviceName: widget.deviceName, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.menka,
          name: S.current.device_cardManage,
          hardware: DeviceFunctionType.card,
          widget: CardListWidget(
              deviceName: widget.deviceName, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.fenxiang,
          name: S.current.device_shareDevice,
          widget: DeviceShareWidget(
              title: S.current.device_shareDevice,
              shareType: ShareType.share,
              deviceId: widget.deviceId,
              deviceName: widget.deviceName)),
      HomeListData(
          icon: WitsIcon.erweima,
          name: S.current.device_shortShare,
          widget: DeviceShareWidget(
              title: S.current.device_shortShare,
              shareType: ShareType.shortShare,
              deviceId: widget.deviceId,
              deviceName: widget.deviceName)),
      HomeListData(
          icon: WitsIcon.jilu,
          name: S.current.device_LockRecord,
          widget: LockRecordWidget(
              deviceName: widget.deviceName, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.zhuanrang,
          name: S.current.device_makeOverDevice,
          widget: MakeOverWidget(
              deviceName: widget.deviceName, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.shebeishezhi,
          name: S.current.device_deviceSettings,
          widget: SlockSetupWidget(
              deviceId: widget.deviceId, deviceName: widget.deviceName)),
      HomeListData(icon: WitsIcon.luyouqi, name: S.current.device_bindRelay),
      HomeListData(icon: WitsIcon.shebei, name: S.current.device_deviceInfo),
      HomeListData(
          icon: WitsIcon.gujianshengji,
          name: S.current.device_ota,
          hardware: DeviceFunctionType.ota),
      HomeListData(icon: WitsIcon.erweima, name: S.current.device_deviceQrCode),
      HomeListData(icon: WitsIcon.saoma, name: S.current.device_scanningCode),
    ];
  }

  ///开门
  _unlock(UnlockState unlockState) async {
    try {
      if (_timer != null) _timer?.cancel();
      unlockState.connect(); //执行正在连接动画
      _updateHintText(S.current.device_connecting); //显示正在连接提示文字
      await BleDevice.instance
          .connectAndOpen(widget.deviceId, _exeLock); //调用执行开门
      if (_exeLock) {
        //true 代表开门成功 false 代表连接成功
        unlockState.openSuccess(); //调用显示成功的动画
        _updateHintText(S.current.device_openLockSuccess(widget.deviceName));
        _timer = Timer(const Duration(seconds: 6), () {
          unlockState.connectSuccess();
          _updateHintText(S.current.device_connected); //显示已连接提示文字
        }); //定时器等待门锁关闭
      } else {
        _exeLock = true; //标记下一次一定是开门
        unlockState.connectSuccess();
        _updateHintText(S.current.device_connected); //显示已连接提示文字
      }
      _listenBleConnectState(unlockState); //调用连接状态监听
    } on PlatformException catch (e) {
      print("错误信息$e");
      _timer = Timer(const Duration(seconds: 5), () {
        unlockState.init();
        _updateHintText(S.current.device_disconnectHint);
      });
      unlockState.openFail();
      WitsystemError? error = AutoErrorHandle.auto(e);
      _updateHintText(error.error);
      setState(() {});
    }
  }

  ///更新页面提示文字
  _updateHintText(String hint) {
    setState(() {
      _hint = hint;
    });
  }

  ///监听和设备的连接状态
  _listenBleConnectState(UnlockState unlockState) async {
    //判断已经监听无须重复监听，防止重复监听
    if (_bleStateListen != null) {
      return;
    }
    Stream _stream = await BleDevice().bleDeviceConnectState();
    _bleStateListen = _stream.listen((value) {
      if (_timer != null) _timer?.cancel();
      // print("状态发生变化1${convert.json.encode(value.toString())}");
      unlockState.init();
      _cancelListenBleConnectState();
      _updateHintText(S.current.device_disconnectHint);
    });
  }

  ///取消监听状态
  _cancelListenBleConnectState() {
    if (_bleStateListen != null) _bleStateListen?.cancel();
    _bleStateListen = null;
  }

  @override
  void dispose() {
    super.dispose();
    BleDevice().closeBleDevice(widget.deviceId);
    if (_timer != null) _timer?.cancel();
    _cancelListenBleConnectState();
  }

  @override
  Widget build(BuildContext context) {
    return HomeTempletWidget(
        list: list!,
        deviceId: widget.deviceId,
        deviceName: widget.deviceName,
        itemOnTap: (HomeListData item, int index) {
          if (item.widget == null) return;
          Navigator.of(context).push(MaterialPageRoute(builder: (context) {
            return item.widget!;
          }));
        },
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: [
            UnlockWidget(
              width: 160,
              height: 160,
              onTop: _unlock,
              immediatelyExecute: true,
            ),
            Container(
              margin: const EdgeInsets.fromLTRB(0, 30, 0, 10),
              child: Text(_hint,
                  style: const TextStyle(color: Colors.white, fontSize: 12)),
            ),
          ],
        ));
  }
}
